@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/*$Id: gdevbbox.c 11578 2010-07-30 23:13:52Z till $ */
+/*$Id: gdevbbox.c 11726 2010-09-17 05:40:48Z mvrhel $ */
 /* Device for tracking bounding box */
 #include "math_.h"
 #include "memory_.h"
@@ -146,7 +146,8 @@ gx_device_bbox gs_bbox_device =
      NULL,			/* ret_devn_params */
      bbox_fillpage,		/* fillpage */
      NULL,                      /* push_transparency_state */
-     NULL                       /* pop_transparency_state */
+     NULL,                      /* pop_transparency_state */
+     NULL                       /* put_image */
     },
     0,				/* target */
     1,				/*true *//* free_standing */
