@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevijs.c 11176 2010-05-04 18:47:12Z robin $ */
+/* $Id: gdevijs.c 12063 2011-01-26 12:25:36Z chrisl $ */
 /*
  * IJS device for Ghostscript.
  * Intended to work with any IJS compliant inkjet driver, including
@@ -66,8 +66,8 @@
 #include <fcntl.h>
 #include "gdevprn.h"
 #include "gp.h"
-#include "ijs.h"
-#include "ijs_client.h"
+#include "ijs/ijs.h"
+#include "ijs/ijs_client.h"
 
 #if 0
 #define KRGB_DEBUG
