@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdf.c 11658 2010-08-26 15:49:51Z ken $ */
+/* $Id: gdevpdf.c 12002 2011-01-06 11:08:52Z ken $ */
 /* PDF-writing driver */
 #include "fcntl_.h"
 #include "memory_.h"
@@ -932,7 +932,7 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
 
     mediabox[2] = round_box_coord(page->MediaBox.x);
     mediabox[3] = round_box_coord(page->MediaBox.y);
-    pdf_open_obj(pdev, page_id);
+    pdf_open_obj(pdev, page_id, resourcePage);
     s = pdev->strm;
     pprintg2(s, "<</Type/Page/MediaBox [0 0 %g %g]\n",
 		mediabox[2], mediabox[3]);
@@ -1044,7 +1044,7 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
     cos_dict_elements_write(page->Page, pdev);
 
     stream_puts(s, ">>\n");
-    pdf_end_obj(pdev);
+    pdf_end_obj(pdev, resourcePage);
     return 0;
 }
 
@@ -1072,6 +1072,8 @@ static int find_end_xref_section (gx_device_pdf *pdev, FILE *tfile, int start, i
 	    ulong pos;
 
 	    r = fread(&pos, sizeof(pos), 1, tfile);
+	    if (r != 1)
+		return(gs_note_error(gs_error_ioerror));
 	    if (pos & ASIDES_BASE_POSITION)
 		pos += resource_pos - ASIDES_BASE_POSITION;
 	    pos -= pdev->OPDFRead_procset_length;
@@ -1096,6 +1098,8 @@ static int write_xref_section(gx_device_pdf *pdev, FILE *tfile, int start, int e
 	    char str[21];
 
 	    r = fread(&pos, sizeof(pos), 1, tfile);
+	    if (r != 1)
+		return(gs_note_error(gs_error_ioerror));
 	    if (pos & ASIDES_BASE_POSITION)
 	        pos += resource_pos - ASIDES_BASE_POSITION;
 	    pos -= pdev->OPDFRead_procset_length;
@@ -1120,7 +1124,7 @@ pdf_close(gx_device * dev)
 	Pages_id = pdev->Pages->id, Encrypt_id = 0;
     long Threads_id = 0;
     bool partial_page = (pdev->contents_id != 0 && pdev->next_page != 0);
-    int code = 0, code1, start_section, end_section;
+    int code = 0, code1, start_section, end_section, pagecount=0;
 
     /*
      * If this is an EPS file, or if the file didn't end with a showpage for
@@ -1140,11 +1144,9 @@ pdf_close(gx_device * dev)
 
     /* Write the page objects. */
 
-    {
-	int i;
-
-	for (i = 1; i <= pdev->next_page; ++i)
-	    pdf_write_page(pdev, i);
+    if (!(pdev->ForOPDFRead && pdev->ProduceDSC)) {
+	for (pagecount = 1; pagecount <= pdev->next_page; ++pagecount)
+	    pdf_write_page(pdev, pagecount);
     }
 
     if (pdev->PrintStatistics)
@@ -1178,150 +1180,154 @@ pdf_close(gx_device * dev)
     code1 = pdf_free_resource_objects(pdev, resourceCMap);
     if (code >= 0)
 	code = code1;
-    if (pdev->ResourcesBeforeUsage)
-	pdf_reverse_resource_chain(pdev, resourcePage);
-    code1 = pdf_write_resource_objects(pdev, resourcePage);
-    if (code >= 0)
-	code = code1;
-    code1 = pdf_free_resource_objects(pdev, resourcePage);
-    if (code >= 0)
-	code = code1;
+    if (!(pdev->ForOPDFRead && pdev->ProduceDSC)) {
+	if (pdev->ResourcesBeforeUsage)
+	    pdf_reverse_resource_chain(pdev, resourcePage);
+	code1 = pdf_write_resource_objects(pdev, resourcePage);
+	if (code >= 0)
+	    code = code1;
+	code1 = pdf_free_resource_objects(pdev, resourcePage);
+	if (code >= 0)
+	    code = code1;
+    }
 
     code1 = pdf_free_resource_objects(pdev, resourceOther);
     if (code >= 0)
 	code = code1;
 
     /* Create the Pages tree. */
+    if (!(pdev->ForOPDFRead && pdev->ProduceDSC)) {
+	pdf_open_obj(pdev, Pages_id, resourcePagesTree);
+	s = pdev->strm;
+	stream_puts(s, "<< /Type /Pages /Kids [\n");
+	/* Omit the last page if it was incomplete. */
+	if (partial_page)
+	    --(pdev->next_page);
+	{
+	    int i;
 
-    pdf_open_obj(pdev, Pages_id);
-    s = pdev->strm;
-    stream_puts(s, "<< /Type /Pages /Kids [\n");
-    /* Omit the last page if it was incomplete. */
-    if (partial_page)
-	--(pdev->next_page);
-    {
-	int i;
+    	    for (i = 0; i < pdev->next_page; ++i) 
+		pprintld1(s, "%ld 0 R\n", pdev->pages[i].Page->id);
+	}
+	pprintd1(s, "] /Count %d\n", pdev->next_page);
+ 
+	/* If the last file was PostScript, its possible that DSC comments might be lying around
+         * and pdf_print_orientation will use that if its present. So make sure we get rid of those
+	 * before considering the dominant page direction for the Pages tree.
+	 */
+	pdev->doc_dsc_info.viewing_orientation = pdev->doc_dsc_info.orientation = -1;
+	pdev->text_rotation.Rotate = pdf_dominant_rotation(&pdev->text_rotation);
+	pdf_print_orientation(pdev, NULL);
 
-	for (i = 0; i < pdev->next_page; ++i) 
-	    pprintld1(s, "%ld 0 R\n", pdev->pages[i].Page->id);
-    }
-    pprintd1(s, "] /Count %d\n", pdev->next_page);
+	cos_dict_elements_write(pdev->Pages, pdev);
+	stream_puts(s, ">>\n");
+	pdf_end_obj(pdev, resourcePagesTree);
 
-    /* If the last file was PostScript, its possible that DSC comments might be lying around
-     * and pdf_print_orientation will use that if its present. So make sure we get rid of those
-     * before considering the dominant page direction for the Pages tree.
-     */
-    pdev->doc_dsc_info.viewing_orientation = pdev->doc_dsc_info.orientation = -1;
-    pdev->text_rotation.Rotate = pdf_dominant_rotation(&pdev->text_rotation);
-    pdf_print_orientation(pdev, NULL);
+	/* Close outlines and articles. */
 
-    cos_dict_elements_write(pdev->Pages, pdev);
-    stream_puts(s, ">>\n");
-    pdf_end_obj(pdev);
-
-    /* Close outlines and articles. */
-
-    if (pdev->outlines_id != 0) {
-	/* depth > 0 is only possible for an incomplete outline tree. */
-	while (pdev->outline_depth > 0) {
-	    code1 = pdfmark_close_outline(pdev);
+	if (pdev->outlines_id != 0) {
+	    /* depth > 0 is only possible for an incomplete outline tree. */
+	    while (pdev->outline_depth > 0) {
+		code1 = pdfmark_close_outline(pdev);
+		if (code >= 0)
+		    code = code1;
+	    }
+	    code = pdfmark_close_outline(pdev);
 	    if (code >= 0)
 		code = code1;
-	}
-	code = pdfmark_close_outline(pdev);
-	if (code >= 0)
-	    code = code1;
-	pdf_open_obj(pdev, pdev->outlines_id);
-	pprintd1(s, "<< /Count %d", pdev->outlines_open);
-	pprintld2(s, " /First %ld 0 R /Last %ld 0 R >>\n",
+	    pdf_open_obj(pdev, pdev->outlines_id, resourceOutline);
+	    pprintd1(s, "<< /Count %d", pdev->outlines_open);
+	    pprintld2(s, " /First %ld 0 R /Last %ld 0 R >>\n",
 		  pdev->outline_levels[0].first.id,
 		  pdev->outline_levels[0].last.id);
-	pdf_end_obj(pdev);
-    }
-    if (pdev->articles != 0) {
-	pdf_article_t *part;
-
-	/* Write the remaining information for each article. */
-	for (part = pdev->articles; part != 0; part = part->next)
-	    pdfmark_write_article(pdev, part);
-    }
-
-    /* Write named destinations.  (We can't free them yet.) */
-
-    if (pdev->Dests)
-	COS_WRITE_OBJECT(pdev->Dests, pdev);
-
-    /* Write the PageLabel array */
-    pdfmark_end_pagelabels(pdev);
-    if (pdev->PageLabels) {
-	COS_WRITE_OBJECT(pdev->PageLabels, pdev);
-    }
-
-    /* Write the document metadata. */
-    code1 = pdf_document_metadata(pdev);
-    if (code >= 0)
-	code = code1;
-
-    /* Write the Catalog. */
-
-    /*
-     * The PDF specification requires Threads to be an indirect object.
-     * Write the threads now, if any.
-     */
-    if (pdev->articles != 0) {
-	pdf_article_t *part;
-
-	Threads_id = pdf_begin_obj(pdev);
-	s = pdev->strm;
-	stream_puts(s, "[ ");
-	while ((part = pdev->articles) != 0) {
-	    pdev->articles = part->next;
-	    pprintld1(s, "%ld 0 R\n", part->contents->id);
-	    COS_FREE(part->contents, "pdf_close(article contents)");
-	    gs_free_object(mem, part, "pdf_close(article)");
+	    pdf_end_obj(pdev, resourceOutline);
 	}
-	stream_puts(s, "]\n");
-	pdf_end_obj(pdev);
-    }
-    pdf_open_obj(pdev, Catalog_id);
-    s = pdev->strm;
-    stream_puts(s, "<<");
-    pprintld1(s, "/Type /Catalog /Pages %ld 0 R\n", Pages_id);
-    if (pdev->outlines_id != 0)
-	pprintld1(s, "/Outlines %ld 0 R\n", pdev->outlines_id);
-    if (Threads_id)
-	pprintld1(s, "/Threads %ld 0 R\n", Threads_id);
-    if (pdev->Dests)
-	pprintld1(s, "/Dests %ld 0 R\n", pdev->Dests->id);
-    if (pdev->PageLabels)
-	pprintld1(s, "/PageLabels << /Nums  %ld 0 R >>\n", 
+	if (pdev->articles != 0) {
+	    pdf_article_t *part;
+
+	    /* Write the remaining information for each article. */
+	    for (part = pdev->articles; part != 0; part = part->next)
+		pdfmark_write_article(pdev, part);
+	}
+
+	/* Write named destinations.  (We can't free them yet.) */
+
+	if (pdev->Dests)
+	    COS_WRITE_OBJECT(pdev->Dests, pdev, resourceDests);
+
+	/* Write the PageLabel array */
+	pdfmark_end_pagelabels(pdev);
+	if (pdev->PageLabels) {
+	    COS_WRITE_OBJECT(pdev->PageLabels, pdev, resourceLabels);
+	}
+
+	/* Write the document metadata. */
+	code1 = pdf_document_metadata(pdev);
+	if (code >= 0)
+	    code = code1;
+
+	/* Write the Catalog. */
+
+	/*
+	 * The PDF specification requires Threads to be an indirect object.
+	 * Write the threads now, if any.
+	 */
+	if (pdev->articles != 0) {
+	    pdf_article_t *part;
+
+	    Threads_id = pdf_begin_obj(pdev, resourceThread);
+	    s = pdev->strm;
+	    stream_puts(s, "[ ");
+	    while ((part = pdev->articles) != 0) {
+		pdev->articles = part->next;
+		pprintld1(s, "%ld 0 R\n", part->contents->id);
+		COS_FREE(part->contents, "pdf_close(article contents)");
+		gs_free_object(mem, part, "pdf_close(article)");
+	    }
+	    stream_puts(s, "]\n");
+	    pdf_end_obj(pdev, resourceThread);
+	}
+	pdf_open_obj(pdev, Catalog_id, resourceCatalog);
+	s = pdev->strm;
+	stream_puts(s, "<<");
+	pprintld1(s, "/Type /Catalog /Pages %ld 0 R\n", Pages_id);
+	if (pdev->outlines_id != 0)
+	    pprintld1(s, "/Outlines %ld 0 R\n", pdev->outlines_id);
+	if (Threads_id)
+	    pprintld1(s, "/Threads %ld 0 R\n", Threads_id);
+	if (pdev->Dests)
+	    pprintld1(s, "/Dests %ld 0 R\n", pdev->Dests->id);
+	if (pdev->PageLabels)
+	    pprintld1(s, "/PageLabels << /Nums  %ld 0 R >>\n", 
                   pdev->PageLabels->id);
-    cos_dict_elements_write(pdev->Catalog, pdev);
-    stream_puts(s, ">>\n");
-    pdf_end_obj(pdev);
-    if (pdev->Dests) {
-	COS_FREE(pdev->Dests, "pdf_close(Dests)");
-	pdev->Dests = 0;
+	cos_dict_elements_write(pdev->Catalog, pdev);
+	stream_puts(s, ">>\n");
+	pdf_end_obj(pdev, resourceCatalog);
+	if (pdev->Dests) {
+	    COS_FREE(pdev->Dests, "pdf_close(Dests)");
+	    pdev->Dests = 0;
+	}
+	if (pdev->PageLabels) {
+	    COS_FREE(pdev->PageLabels, "pdf_close(PageLabels)");
+	    pdev->PageLabels = 0;
+	    pdev->PageLabels_current_label = 0;
+	}
+
+	/* Prevent writing special named objects twice. */
+
+	pdev->Catalog->id = 0;
+	/*pdev->Info->id = 0;*/	/* Info should get written */
+	pdev->Pages->id = 0;
+	{
+	    int i;
+
+	    for (i = 0; i < pdev->num_pages; ++i)
+		if (pdev->pages[i].Page)
+		    pdev->pages[i].Page->id = 0;
+	}
+    } else {
+	pdev->Info->id = 0;	/* Don't write Info dict for DSC PostScript */
     }
-    if (pdev->PageLabels) {
-	COS_FREE(pdev->PageLabels, "pdf_close(PageLabels)");
-	pdev->PageLabels = 0;
-        pdev->PageLabels_current_label = 0;
-    }
-
-    /* Prevent writing special named objects twice. */
-
-    pdev->Catalog->id = 0;
-    /*pdev->Info->id = 0;*/	/* Info should get written */
-    pdev->Pages->id = 0;
-    {
-	int i;
-
-	for (i = 0; i < pdev->num_pages; ++i)
-	    if (pdev->pages[i].Page)
-		pdev->pages[i].Page->id = 0;
-    }
-
     /*
      * Write the definitions of the named objects.
      * Note that this includes Form XObjects created by BP/EP, named PS
@@ -1333,6 +1339,15 @@ pdf_close(gx_device * dev)
     } while (pdf_pop_namespace(pdev) >= 0);
     cos_dict_objects_write(pdev->global_named_objects, pdev);
 
+    if (pdev->ForOPDFRead && pdev->ProduceDSC) {
+	int pages;
+
+	for (pages = 0; pages <= pdev->next_page; ++pages)
+	    ;
+
+	code = ps2write_dsc_header(pdev, pages - 1);
+    }
+
     /* Copy the resources into the main file. */
 
     s = pdev->strm;
@@ -1346,64 +1361,106 @@ pdf_close(gx_device * dev)
 	pdf_copy_data(s, rfile, res_end, NULL);
     }
 
-    /* Write Encrypt. */
-    if (pdev->OwnerPassword.size > 0) {
-	Encrypt_id = pdf_obj_ref(pdev);
+    if (pdev->ForOPDFRead && pdev->ProduceDSC) {
+	int j;
+	
+	code = 0;
+	pagecount = 1;
 
-	pdf_open_obj(pdev, Encrypt_id);
-	s = pdev->strm;
-	stream_puts(s, "<<");
-	stream_puts(s, "/Filter /Standard ");
-	pprintld1(s, "/V %ld ", pdev->EncryptionV);
-	pprintld1(s, "/Length %ld ", pdev->KeyLength);
-	pprintld1(s, "/R %ld ", pdev->EncryptionR);
-	pprintld1(s, "/P %ld ", pdev->Permissions);
-	stream_puts(s, "/O ");
-	pdf_put_string(pdev, pdev->EncryptionO, sizeof(pdev->EncryptionO));
-	stream_puts(s, "\n/U ");
-	pdf_put_string(pdev, pdev->EncryptionU, sizeof(pdev->EncryptionU));
-	stream_puts(s, ">>\n");
-	pdf_end_obj(pdev);
-	s = pdev->strm;
+	/* All resources and procsets written, end the prolog */
+	stream_puts(pdev->strm, "%%EndProlog\n");
+
+	if (pdev->ResourcesBeforeUsage)
+	    pdf_reverse_resource_chain(pdev, resourcePage);
+	for (j = 0; j < NUM_RESOURCE_CHAINS && code >= 0; ++j) {
+	    pdf_resource_t *pres = pdev->resources[resourcePage].chains[j];
+
+	    for (; pres != 0; pres = pres->next)
+		if ((!pres->named || pdev->ForOPDFRead) 
+		    && !pres->object->written) {
+		    pdf_page_t *page = &pdev->pages[pagecount - 1];
+
+		    pprintd2(pdev->strm, "%%%%Page: %d %d\n",
+			pagecount, pagecount);
+		    pprintd2(pdev->strm, "%%%%PageBoundingBox: 0 0 %d %d\n", (int)page->MediaBox.x, (int)page->MediaBox.y);
+		    stream_puts(pdev->strm, "%%BeginPageSetup\n");
+		    stream_puts(pdev->strm, "/pagesave save def\n");
+		    pdf_write_page(pdev, pagecount++);
+		    stream_puts(pdev->strm, "%%EndPageSetup\n");
+		    pprintld1(pdev->strm, "%ld 0 obj\n", pres->object->id);
+		    code = cos_write(pres->object, pdev, pres->object->id);
+		    stream_puts(pdev->strm, "endobj\n");
+		    pres->object->written = true;
+		    stream_puts(pdev->strm, "pagesave restore\n%%PageTrailer\n");
+		}
+	}
+	code1 = pdf_free_resource_objects(pdev, resourcePage);
+	if (code >= 0)
+	    code = code1;
+	stream_puts(pdev->strm, "%%Trailer\n");
+	stream_puts(pdev->strm, "%%EOF\n");
     }
 
-    /* Write the cross-reference section. */
+    if (!(pdev->ForOPDFRead && pdev->ProduceDSC)) {
+	/* Write Encrypt. */
+	if (pdev->OwnerPassword.size > 0) {
+	    Encrypt_id = pdf_obj_ref(pdev);
 
-    start_section = pdev->FirstObjectNumber;
-    end_section = find_end_xref_section(pdev, tfile, start_section, resource_pos);
+	    pdf_open_obj(pdev, Encrypt_id, resourceEncrypt);
+	    s = pdev->strm;
+	    stream_puts(s, "<<");
+	    stream_puts(s, "/Filter /Standard ");
+	    pprintld1(s, "/V %ld ", pdev->EncryptionV);
+	    pprintld1(s, "/Length %ld ", pdev->KeyLength);
+	    pprintld1(s, "/R %ld ", pdev->EncryptionR);
+	    pprintld1(s, "/P %ld ", pdev->Permissions);
+	    stream_puts(s, "/O ");
+	    pdf_put_string(pdev, pdev->EncryptionO, sizeof(pdev->EncryptionO));
+	    stream_puts(s, "\n/U ");
+	    pdf_put_string(pdev, pdev->EncryptionU, sizeof(pdev->EncryptionU));
+	    stream_puts(s, ">>\n");
+	    pdf_end_obj(pdev, resourceEncrypt);
+	    s = pdev->strm;
+	}
 
-    xref = pdf_stell(pdev) - pdev->OPDFRead_procset_length;
-    if (pdev->FirstObjectNumber == 1)
-	pprintld1(s, "xref\n0 %ld\n0000000000 65535 f \n",
+	/* Write the cross-reference section. */
+
+	start_section = pdev->FirstObjectNumber;
+	end_section = find_end_xref_section(pdev, tfile, start_section, resource_pos);
+
+	xref = pdf_stell(pdev) - pdev->OPDFRead_procset_length;
+	if (pdev->FirstObjectNumber == 1)
+	    pprintld1(s, "xref\n0 %ld\n0000000000 65535 f \n",
 		  end_section);
-    else
-	pprintld2(s, "xref\n0 1\n0000000000 65535 f \n%ld %ld\n",
+	else
+	    pprintld2(s, "xref\n0 1\n0000000000 65535 f \n%ld %ld\n",
 		  start_section,
 		  end_section - start_section);
 
-    do {
-	write_xref_section(pdev, tfile, start_section, end_section, resource_pos);
-	if (end_section >= pdev->next_id)
-	    break;
-	start_section = end_section + 1;
-	end_section = find_end_xref_section(pdev, tfile, start_section, resource_pos);
-	pprintld2(s, "%ld %ld\n", start_section, end_section - start_section);
-    } while (1);
+	do {
+	    write_xref_section(pdev, tfile, start_section, end_section, resource_pos);
+	    if (end_section >= pdev->next_id)
+		break;
+	    start_section = end_section + 1;
+	    end_section = find_end_xref_section(pdev, tfile, start_section, resource_pos);
+	    pprintld2(s, "%ld %ld\n", start_section, end_section - start_section);
+	} while (1);
 
-    /* Write the trailer. */
+	/* Write the trailer. */
 
-    stream_puts(s, "trailer\n");
-    pprintld3(s, "<< /Size %ld /Root %ld 0 R /Info %ld 0 R\n",
+	stream_puts(s, "trailer\n");
+	pprintld3(s, "<< /Size %ld /Root %ld 0 R /Info %ld 0 R\n",
 	      pdev->next_id, Catalog_id, Info_id);
-    stream_puts(s, "/ID [");
-    psdf_write_string(pdev->strm, pdev->fileID, sizeof(pdev->fileID), 0);
-    psdf_write_string(pdev->strm, pdev->fileID, sizeof(pdev->fileID), 0);
-    stream_puts(s, "]\n");
-    if (pdev->OwnerPassword.size > 0) {
-	pprintld1(s, "/Encrypt %ld 0 R ", Encrypt_id);
+	stream_puts(s, "/ID [");
+	psdf_write_string(pdev->strm, pdev->fileID, sizeof(pdev->fileID), 0);
+	psdf_write_string(pdev->strm, pdev->fileID, sizeof(pdev->fileID), 0);
+	stream_puts(s, "]\n");
+	if (pdev->OwnerPassword.size > 0) {
+	    pprintld1(s, "/Encrypt %ld 0 R ", Encrypt_id);
+	}
+	stream_puts(s, ">>\n");
+	pprintld1(s, "startxref\n%ld\n%%%%EOF\n", xref);
     }
-    stream_puts(s, ">>\n");
-    pprintld1(s, "startxref\n%ld\n%%%%EOF\n", xref);
 
     /* Release the resource records. */
 
@@ -1438,7 +1495,8 @@ pdf_close(gx_device * dev)
            Removing them now. */
 	int status;
     
-	stream_putc(s, 0x04);
+	if (!pdev->ProduceDSC)
+	    stream_putc(s, 0x04);
 	while (s->strm) {
 	    s = s->strm;
 	}
@@ -1446,6 +1504,7 @@ pdf_close(gx_device * dev)
 	if (status < 0 && code == 0)
 	    code = gs_error_ioerror;
     }
+
     code1 = gdev_vector_close_file((gx_device_vector *) pdev);
     if (code >= 0)
 	code = code1;
