@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfe.c 11649 2010-08-23 16:49:47Z ken $ */
+/* $Id: gdevpdfe.c 11950 2010-12-13 11:57:36Z ken $ */
 /* Metadata writer. */
 #include "gx.h"
 #include "gserrors.h"
@@ -256,7 +256,7 @@ pdf_xmp_convert_time(char *dt, int dtl, char *buf, int bufl)
     return 25;
 }
 	
-static int
+int
 pdf_get_docinfo_item(gx_device_pdf *pdev, const char *key, char *buf, int buf_length)
 {
     const cos_value_t *v = cos_dict_find(pdev->Info, (const byte *)key, strlen(key));
@@ -780,7 +780,7 @@ pdf_document_metadata(gx_device_pdf *pdev)
 	s_MD5C_get_digest(pdev->strm, digest, sizeof(digest));
 	if (pdev->EncryptMetadata)
 	    options |= DATA_STREAM_ENCRYPT;
-	code = pdf_open_aside(pdev, resourceOther, gs_no_id, &pres, true, options);
+	code = pdf_open_aside(pdev, resourceMetadata, gs_no_id, &pres, true, options);
 	if (code < 0)
 	    return code;
 	code = cos_dict_put_c_key_string((cos_dict_t *)pres->object, "/Type", (const byte *)"/Metadata", 9);
@@ -795,7 +795,7 @@ pdf_document_metadata(gx_device_pdf *pdev)
 	code = pdf_close_aside(pdev);
 	if (code < 0)
 	    return code;
-	code = COS_WRITE_OBJECT(pres->object, pdev);
+	code = COS_WRITE_OBJECT(pres->object, pdev, resourceNone);
 	if (code < 0)
 	    return code;
 	sprintf(buf, "%ld 0 R", pres->object->id);
