@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfg.c 11346 2010-06-01 17:41:20Z mvrhel $ */
+/* $Id: gdevpdfg.c 12168 2011-02-18 11:15:23Z ken $ */
 /* Graphics state management for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -409,7 +409,7 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 		return code;
 	    pprints1(pdev->strm, " %s\n", command);
 	    break;
-	case pattern_color_sapce:
+	case pattern_color_space:
 	    {	pdf_resource_t *pres;
 
 		if (pdc->type == gx_dc_type_pattern)
@@ -886,7 +886,7 @@ pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
     char trs[17 + MAX_FN_CHARS + 1];
     int code = pdf_write_transfer(pdev, porder->transfer, "/TransferFunction",
 				  trs);
-    long id, spot_id;
+    long spot_id;
     stream *s;
     int i = countof(ht_functions);
     gs_memory_t *mem = pdev->pdf_memory;
@@ -935,7 +935,7 @@ pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
 	if (code < 0) 
 	    return code;
     }	
-    *pid = id = pdf_begin_separate(pdev);
+    *pid = pdf_begin_separate(pdev, resourceHalftone);
     s = pdev->strm;
     /* Use the original, requested frequency and angle. */
     pprintg2(s, "<</Type/Halftone/HalftoneType 1/Frequency %g/Angle %g",
@@ -948,7 +948,7 @@ pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
     if (psht->accurate_screens)
 	stream_puts(s, "/AccurateScreens true");
     stream_puts(s, ">>\n");
-    return pdf_end_separate(pdev);
+    return pdf_end_separate(pdev, resourceHalftone);
 }
 static int
 pdf_write_screen_halftone(gx_device_pdf *pdev, const gs_screen_halftone *psht,
@@ -978,7 +978,7 @@ pdf_write_colorscreen_halftone(gx_device_pdf *pdev,
 	if (code < 0)
 	    return code;
     }
-    *pid = pdf_begin_separate(pdev);
+    *pid = pdf_begin_separate(pdev, resourceHalftone);
     s = pdev->strm;
     /* Use Black, Gray as the Default unless we are in RGB colormodel */
     /* (num_comp < 4) in which case we use Green (arbitrarily) */
@@ -990,7 +990,7 @@ pdf_write_colorscreen_halftone(gx_device_pdf *pdev,
     if (pdht->num_comp > 3)
     pprintld2(s, "/Gray %ld 0 R/Black %ld 0 R", ht_ids[3], ht_ids[3]);
     stream_puts(s, ">>\n");
-    return pdf_end_separate(pdev);
+    return pdf_end_separate(pdev, resourceHalftone);
 }
 
 #define CHECK(expr)\
@@ -1002,7 +1002,6 @@ pdf_write_threshold_halftone(gx_device_pdf *pdev,
 			     const gx_ht_order *porder, long *pid)
 {
     char trs[17 + MAX_FN_CHARS + 1];
-    stream *s;
     pdf_data_writer_t writer;
     int code = pdf_write_transfer(pdev, porder->transfer, "",
 				  trs);
@@ -1010,7 +1009,6 @@ pdf_write_threshold_halftone(gx_device_pdf *pdev,
     if (code < 0)
 	return code;
     CHECK(pdf_begin_data(pdev, &writer)); 
-    s = pdev->strm;
     *pid = writer.pres->object->id;
     CHECK(cos_dict_put_c_strings((cos_dict_t *)writer.pres->object,
 	"/Type", "/Halftone"));
@@ -1040,7 +1038,6 @@ pdf_write_threshold2_halftone(gx_device_pdf *pdev,
     if (code < 0)
 	return code;
     CHECK(pdf_begin_data(pdev, &writer)); 
-    s = pdev->strm;
     *pid = writer.pres->object->id;
     CHECK(cos_dict_put_c_strings((cos_dict_t *)writer.pres->object,
 	"/Type", "/Halftone"));
@@ -1147,7 +1144,7 @@ pdf_write_multiple_halftone(gx_device_pdf *pdev,
 	    return code;
 	}
     }
-    *pid = pdf_begin_separate(pdev);
+    *pid = pdf_begin_separate(pdev, resourceHalftone);
     s = pdev->strm;
     stream_puts(s, "<</Type/Halftone/HalftoneType 5\n");
     done_Default = false;
@@ -1187,7 +1184,7 @@ pdf_write_multiple_halftone(gx_device_pdf *pdev,
     }
     stream_puts(s, ">>\n");
     gs_free_object(mem, ids, "pdf_write_multiple_halftone");
-    return pdf_end_separate(pdev);
+    return pdf_end_separate(pdev, resourceHalftone);
 }
 
 /*
