@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfj.c 11490 2010-07-06 07:04:53Z ken $ */
+/* $Id: gdevpdfj.c 12171 2011-02-18 11:37:56Z ken $ */
 /* Image-writing utilities for pdfwrite driver */
 #include "memory_.h"
 #include "gx.h"
@@ -419,14 +419,14 @@ pdf_complete_image_data(gx_device_pdf *pdev, pdf_image_writer *piw, int data_h,
 	    int lines_left = piw->height - data_h;
 	    byte buf[256];
 	    const uint lb = sizeof(buf);
-	    int i, l, status;
+	    int i, l;
 	    uint ignore;
 
 	    memset(buf, 128, lb);
 	    for (; lines_left; lines_left--) 
 		for (i = 0; i < piw->alt_writer_count; i++) {
 		    for (l = bytes_per_line; l > 0; l -= lb)
-			if ((status = sputs(piw->binary[i].strm, buf, min(l, lb), 
+			if ((sputs(piw->binary[i].strm, buf, min(l, lb), 
 					    &ignore)) < 0)
 			    return_error(gs_error_ioerror);
 		}
@@ -515,9 +515,24 @@ pdf_end_write_image(gx_device_pdf * pdev, pdf_image_writer * piw)
 	    *(cos_object_t *)named = *pco;
 	    pres->object = COS_OBJECT(named);
 	} else if (!pres->named) { /* named objects are written at the end */
-	    code = pdf_substitute_resource(pdev, &piw->pres, resourceXObject, NULL, false);
-	    if (code < 0)
-		return code;
+	    if (pdev->DetectDuplicateImages) {
+		pdf_x_object_t *pxo = (pdf_x_object_t *)piw->pres;
+		int height = pxo->height, width = pxo->width;
+
+		code = pdf_substitute_resource(pdev, &piw->pres, resourceXObject, NULL, false);
+		if (code < 0)
+		    return code;
+
+		/* These values are related to the image matrix and should *not* be
+		 * substituted if we found a duplicate image, or the matrix calculation
+		 * will be incorrect! This only seems to matter for the PCL interpreter.
+		 */
+		pxo = (pdf_x_object_t *)piw->pres;
+		pxo->height = height;		
+		pxo->width = width;		
+	    } else {
+		pdf_reserve_object_id(pdev, piw->pres, gs_no_id);
+	    }
 	    /*  Warning : If the substituted image used alternate streams,
 		its space in the pdev->streams.strm file won't be released. */
 	    piw->pres->where_used |= pdev->used_mask;
