@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfk.c 11671 2010-09-01 13:59:34Z ken $ */
+/* $Id: gdevpdfk.c 11946 2010-12-10 13:26:11Z ken $ */
 /* Lab and ICCBased color space writing */
 #include "math_.h"
 #include "memory_.h"
@@ -351,7 +351,7 @@ pdf_finish_iccbased(cos_stream_t *pcstrm)
     gx_device_pdf *pdev = pcstrm->pdev;
 
     pcstrm->id = pdf_obj_ref(pdev);
-    return cos_write_object(COS_OBJECT(pcstrm), pdev);
+    return cos_write_object(COS_OBJECT(pcstrm), pdev, resourceICC);
 }
 
 /*
