@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfp.c 11176 2010-05-04 18:47:12Z robin $ */
+/* $Id: gdevpdfp.c 12286 2011-03-13 11:57:08Z ken $ */
 /* Get/put parameters for PDF-writing driver */
 #include "memory_.h"
 #include "gx.h"
@@ -84,6 +84,7 @@ static const gs_param_item_t pdf_param_items[] = {
 
 	/* Target viewer capabilities (Ghostscript-specific)  */
  /* pi("ForOPDFRead", gs_param_type_bool, ForOPDFRead),			    pdfwrite-only */
+    pi("ProduceDSC", gs_param_type_bool, ProduceDSC),
     pi("PatternImagemask", gs_param_type_bool, PatternImagemask),
     pi("MaxClipPathSize", gs_param_type_int, MaxClipPathSize),
     pi("MaxShadingBitmapSize", gs_param_type_int, MaxShadingBitmapSize),
@@ -113,6 +114,14 @@ static const gs_param_item_t pdf_param_items[] = {
     pi("PreserveSeparation", gs_param_type_bool, PreserveSeparation),
     pi("PreserveDeviceN", gs_param_type_bool, PreserveDeviceN),
     pi("PDFACompatibilityPolicy", gs_param_type_int, PDFACompatibilityPolicy),
+    pi("DetectDuplicateImages", gs_param_type_bool, DetectDuplicateImages),
+    pi("AllowIncrementalCFF", gs_param_type_bool, AllowIncrementalCFF),
+    pi("HighLevelDevice", gs_param_type_bool, HighLevelDevice),
+    pi("WantsToUnicode", gs_param_type_bool, WantsToUnicode),
+    pi("AllowPSRepeatFunctions", gs_param_type_bool, AllowPSRepeatFunctions),
+    pi("IsDistiller", gs_param_type_bool, IsDistiller),
+    pi("PreserveSMask", gs_param_type_bool, PreserveSMask),
+    pi("PreserveTrMode", gs_param_type_bool, PreserveTrMode),
 #undef pi
     gs_param_item_end
 };
@@ -211,8 +220,9 @@ gdev_pdf_get_params(gx_device * dev, gs_param_list * plist)
 	 (code = param_write_null(plist, "pdfmark")) < 0) ||
 	(param_requested(plist, "DSC") > 0 &&
 	 (code = param_write_null(plist, "DSC")) < 0) ||
-	(code = gs_param_write_items(plist, pdev, NULL, pdf_param_items)) < 0
-	);
+	(code = gs_param_write_items(plist, pdev, NULL, pdf_param_items)) < 0 
+	)
+    {}
     return code;
 }
 
@@ -281,6 +291,8 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
      * psdf_put_params.
      */
     ecode = code = param_read_bool(plist, "LockDistillerParams", &locked);
+    if (ecode < 0)
+	param_signal_error(plist, param_name, ecode);
  
     if (!(locked && pdev->params.LockDistillerParams)) {
 	/* General parameters. */
@@ -289,6 +301,8 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
 	    int efo = 1;
 
 	    ecode = param_put_int(plist, (param_name = ".EmbedFontObjects"), &efo, ecode);
+	    if (ecode < 0)
+		param_signal_error(plist, param_name, ecode);
 	    if (efo != 1)
 		param_signal_error(plist, param_name, ecode = gs_error_rangecheck);
 	}
@@ -296,6 +310,8 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
 	    int cdv = CoreDistVersion;
 
 	    ecode = param_put_int(plist, (param_name = "CoreDistVersion"), &cdv, ecode);
+	    if (ecode < 0)
+		return gs_note_error(ecode);
 	    if (cdv != CoreDistVersion)
 		param_signal_error(plist, param_name, ecode = gs_error_rangecheck);
 	}
@@ -317,8 +333,12 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
 		    cl = (float)1.3;
 		else if (cl < (float)1.45)
 		    cl = (float)1.4;
-		else
+		else if (cl < (float)1.55)
 		    cl = (float)1.5;
+		else if (cl < (float)1.65)
+		    cl = (float)1.6;
+		else
+		    cl = (float)1.7;
 	    case 1:
 		break;
 	}
@@ -341,7 +361,8 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
 	    if (code < 0 ||
 		(pdev->is_ps2write && (code = param_read_string(plist, "OPDFReadProcsetPath", &pdev->OPDFReadProcsetPath)) < 0) ||
 		(!pdev->is_ps2write && (code = param_read_bool(plist, "ForOPDFRead", &pdev->ForOPDFRead)) < 0)
-		);
+		){
+	    }
 	    plist->memory = mem;
 	}
 	if (code < 0)
@@ -386,6 +407,11 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     }
     if (ecode < 0)
 	goto fail;
+
+    if (pdev->is_ps2write && (code = param_read_bool(plist, "ProduceDSC", &pdev->ProduceDSC)) < 0) {
+	param_signal_error(plist, param_name, code);
+    }
+
     /* PDFA and PDFX are stored in the page device dictionary and therefore
      * set on every setpagedevice. However, if we have encountered a file which
      * can't be made this way, and the PDFACompatibilityPolicy is 1, we want to
@@ -440,6 +466,9 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
 	pdev->HaveStrokeColor = true;
     }
     pdev->ParamCompatibilityLevel = cl;
+    if (cl < 1.2) {
+	pdev->HaveCFF = false;
+    }
     ecode = gdev_psdf_put_params(dev, plist);
     if (ecode < 0)
 	goto fail;
@@ -538,6 +567,19 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     }
     /* Handle the float/double mismatch. */
     pdev->CompatibilityLevel = (int)(cl * 10 + 0.5) / 10.0;
+    if(pdev->OwnerPassword.size != save_dev->OwnerPassword.size || 
+	(pdev->OwnerPassword.size != 0 && 
+	 memcmp(pdev->OwnerPassword.data, save_dev->OwnerPassword.data, 
+	 pdev->OwnerPassword.size) != 0)) {
+	if (pdev->is_open) {
+	    if (pdev->PageCount == 0) {
+	        gs_closedevice((gx_device *)save_dev);
+	        return 0;
+	    }
+	    else
+	        emprintf(pdev->memory, "Owner Password changed mid-job, ignoring.\n");
+	}
+    }
     return 0;
  fail:
     /* Restore all the parameters to their original state. */
