@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfx.h 11632 2010-08-13 19:46:28Z ken $ */
+/* $Id: gdevpdfx.h 12286 2011-03-13 11:57:08Z ken $ */
 /* Internal definitions for PDF-writing driver. */
 
 #ifndef gdevpdfx_INCLUDED
@@ -99,7 +99,10 @@ typedef enum {
     resourcePattern,
     resourceShading,
     resourceXObject,
-    resourceOther, /* Anything else that needs to be stored for a time. */
+    resourceOther, /* Anything else that needs to be stored for a time. 
+		    * Can be any of the types defined below NUM_RESOURCE_TYPES
+		    * but this is the type used to identify the object.
+		    */
     resourceFont,
     /*
      * Internally used (pseudo-)resources.
@@ -112,12 +115,37 @@ typedef enum {
     resourceSoftMaskDict,
     resourceFunction,
     resourcePage,
-    NUM_RESOURCE_TYPES
+    NUM_RESOURCE_TYPES,
+    /* These resource types were 'resourceOther', but we want to track them
+     * for ps2write. They are not stored in the pdf device structure, unlike
+     * the reource types above.
+     */
+    resourceEncoding,
+    resourceCIDSystemInfo,
+    resourceHalftone,
+    resourceLength,
+    resourceStream,
+    resourceOutline,
+    resourceArticle,
+    resourceDests,
+    resourceLabels,
+    resourceThread,
+    resourceCatalog,
+    resourceEncrypt,
+    resourcePagesTree,
+    resourceMetadata,
+    resourceICC,
+    resourceAnnotation,
+    resourceNone	/* Special, used when this isn't a resource at all 
+			 * eg when we execute a resource we've just written, such as
+			 * a Pattern.
+			 */
 } pdf_resource_type_t;
 
 #define PDF_RESOURCE_TYPE_NAMES\
   "/ColorSpace", "/ExtGState", "/Pattern", "/Shading", "/XObject", 0, "/Font",\
-  0, "/Font", "/CMap", "/FontDescriptor", "/Group", "/Mask", 0, 0
+  0, "/Font", "/CMap", "/FontDescriptor", "/Group", "/Mask", 0, 0, 0, 0, 0,\
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 #define PDF_RESOURCE_TYPE_STRUCTS\
   &st_pdf_color_space,		/* gdevpdfg.h / gdevpdfc.c */\
   &st_pdf_resource,		/* see below */\
@@ -436,6 +464,7 @@ struct gx_device_pdf_s {
     bool ResourcesBeforeUsage; /* PS2WRITE only. */
     bool HavePDFWidths;        /* PS2WRITE only. */
     bool HaveStrokeColor;      /* PS2WRITE only. */
+    bool ProduceDSC;	       /* PS2WRITE only. */
     bool HaveTransparency;
     bool PatternImagemask; /* The target viewer|printer handles imagemask 
 			      with pattern color. */
@@ -668,6 +697,14 @@ struct gx_device_pdf_s {
     bool PreserveSeparation;
     bool PreserveDeviceN;
     int PDFACompatibilityPolicy;
+    bool DetectDuplicateImages;
+    bool AllowIncrementalCFF;
+    bool HighLevelDevice;
+    bool WantsToUnicode;
+    bool AllowPSRepeatFunctions;
+    bool IsDistiller;
+    bool PreserveSMask;
+    bool PreserveTrMode;
 };
 
 #define is_in_page(pdev)\
@@ -692,8 +729,9 @@ struct gx_device_pdf_s {
  m(28,sbstack) m(29,substream_Resources) m(30,font3)\
  m(31,accumulating_substream_resource) \
  m(32,pres_soft_mask_dict) m(33,PDFXTrimBoxToMediaBoxOffset.data)\
- m(34,PDFXBleedBoxToTrimBoxOffset.data) m(35, DSCEncodingToUnicode.data)
-#define gx_device_pdf_num_ptrs 36
+ m(34,PDFXBleedBoxToTrimBoxOffset.data) m(35, DSCEncodingToUnicode.data)\
+ m(36,Identity_ToUnicode_CMaps[0]) m(37,Identity_ToUnicode_CMaps[1])
+#define gx_device_pdf_num_ptrs 38
 #define gx_device_pdf_do_param_strings(m)\
     m(0, OPDFReadProcsetPath) m(1, OwnerPassword) m(2, UserPassword) m(3, NoEncrypt)\
     m(4, DocumentUUID) m(5, InstanceUUID)
@@ -761,6 +799,9 @@ void pdf_reset_text(gx_device_pdf *pdev);
 
 /* ------ Document ------ */
 
+/* Write a DSC compliant header to the file */
+int ps2write_dsc_header(gx_device_pdf * pdev, int pages);
+
 /* Open the document if necessary. */
 int pdf_open_document(gx_device_pdf * pdev);
 
@@ -776,11 +817,11 @@ long pdf_obj_ref(gx_device_pdf * pdev);
 long pdf_stell(gx_device_pdf * pdev);
 
 /* Begin an object, optionally allocating an ID. */
-long pdf_open_obj(gx_device_pdf * pdev, long id);
-long pdf_begin_obj(gx_device_pdf * pdev);
+long pdf_open_obj(gx_device_pdf * pdev, long id, pdf_resource_type_t type);
+long pdf_begin_obj(gx_device_pdf * pdev, pdf_resource_type_t type);
 
 /* End an object. */
-int pdf_end_obj(gx_device_pdf * pdev);
+int pdf_end_obj(gx_device_pdf * pdev, pdf_resource_type_t type);
 
 /* ------ Page contents ------ */
 
@@ -805,8 +846,8 @@ extern const gs_memory_struct_type_t *const pdf_resource_type_structs[];
 
 /* Begin an object logically separate from the contents. */
 /* (I.e., an object in the resource file.) */
-long pdf_open_separate(gx_device_pdf * pdev, long id);
-long pdf_begin_separate(gx_device_pdf * pdev);
+long pdf_open_separate(gx_device_pdf * pdev, long id, pdf_resource_type_t type);
+long pdf_begin_separate(gx_device_pdf * pdev, pdf_resource_type_t type);
 
 /* Reserve object id. */
 void pdf_reserve_object_id(gx_device_pdf * pdev, pdf_resource_t *ppres, long id);
@@ -818,7 +859,7 @@ int pdf_alloc_aside(gx_device_pdf * pdev, pdf_resource_t ** plist,
 /* Begin an aside (resource, annotation, ...). */
 int pdf_begin_aside(gx_device_pdf * pdev, pdf_resource_t **plist,
 		    const gs_memory_struct_type_t * pst,
-		    pdf_resource_t **ppres);
+		    pdf_resource_t **ppres, pdf_resource_type_t type);
 
 /* Begin a resource of a given type. */
 int pdf_begin_resource(gx_device_pdf * pdev, pdf_resource_type_t rtype,
@@ -872,13 +913,13 @@ int pdf_substitute_resource(gx_device_pdf *pdev, pdf_resource_t **ppres,
 long pdf_resource_id(const pdf_resource_t *pres);
 
 /* End a separate object. */
-int pdf_end_separate(gx_device_pdf * pdev);
+int pdf_end_separate(gx_device_pdf * pdev, pdf_resource_type_t type);
 
 /* End an aside. */
-int pdf_end_aside(gx_device_pdf * pdev);
+int pdf_end_aside(gx_device_pdf * pdev, pdf_resource_type_t type);
 
 /* End a resource. */
-int pdf_end_resource(gx_device_pdf * pdev);
+int pdf_end_resource(gx_device_pdf * pdev, pdf_resource_type_t type);
 
 /*
  * Write the Cos objects for resources local to a content stream.
