@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtc.c 11176 2010-05-04 18:47:12Z robin $ */
+/* $Id: gdevpdtc.c 11770 2010-10-07 12:49:54Z ken $ */
 /* Composite and CID-based text processing for pdfwrite. */
 #include "memory_.h"
 #include "gx.h"
@@ -488,7 +488,6 @@ scan_cmap_text(pdf_text_enum_t *pte, void *vbuf)
 		    return_error(gs_error_unregistered); /* Must not happen */
 		if (pdsubf->FontType == ft_user_defined  || pdsubf->FontType == ft_encrypted || 
 				pdsubf->FontType == ft_encrypted2) {
-		    code += 0; /* A good place for a breakpoint. */
 		} else {
 		    pdf_font_resource_t *pdfont;
 		    bool notdef_subst = false;
@@ -702,7 +701,6 @@ scan_cmap_text(pdf_text_enum_t *pte, void *vbuf)
 		    type1_glyphs[0] = type1_glyphs[num_type1_glyphs];
 		    num_type1_glyphs = 1;
 		    subfont = saved_subfont;
-		    font_change = 0;
 		} else {
 		    num_type1_glyphs = 0;
 		}
