@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtd.c 11097 2010-04-21 18:12:56Z robin $ */
+/* $Id: gdevpdtd.c 12086 2011-02-01 08:34:27Z ken $ */
 /* FontDescriptor implementation for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
@@ -163,7 +163,7 @@ write_FontDescriptor_common(gx_device_pdf *pdev,
     printer_param_list_t rlist;
     gs_param_list *const plist = (gs_param_list *)&rlist;
 
-    pdf_open_separate(pdev, pdf_font_descriptor_common_id(pfd));
+    pdf_open_separate(pdev, pdf_font_descriptor_common_id(pfd), resourceFontDescriptor);
     s = pdev->strm;
     stream_puts(s, "<</Type/FontDescriptor/FontName");
     pdf_put_name(pdev, pfd->values.FontName.data, pfd->values.FontName.size);
@@ -403,7 +403,7 @@ pdf_compute_font_descriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
      */
     notdef = GS_NO_GLYPH;
     for (index = 0;
-	 (code = bfont->procs.enumerate_glyph((gs_font *)bfont, &index, 
+	 (bfont->procs.enumerate_glyph((gs_font *)bfont, &index, 
 		(is_cid ? GLYPH_SPACE_INDEX : GLYPH_SPACE_NAME), &glyph)) >= 0 &&
 	     index != 0;
 	 ) {
@@ -639,7 +639,7 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_resource_t *pres)
 	pdf_font_descriptor_common_t fd;
 
 	fd = pfd->common;
-	if (pfd->embed && pfd->FontType == ft_TrueType && !pdev->PDFA &&
+	if (pfd->embed && pfd->FontType == ft_TrueType /*&& !pdev->PDFA*/ &&
 	    pdf_do_subset_font(pdev, pfd->base_font, pfd->common.rid)
 	    )
 	    fd.values.Flags =
@@ -676,11 +676,11 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_resource_t *pres)
 	COS_WRITE(pfd->cid.FD, pdev);
     }
     stream_puts(s, ">>\n");
-    pdf_end_separate(pdev);
+    pdf_end_separate(pdev, resourceFontDescriptor);
     pfd->common.object->written = true;
     {	const cos_object_t *pco = (const cos_object_t *)pdf_get_FontFile_object(pfd->base_font);
 	if (pco != NULL) {
-	    code = COS_WRITE_OBJECT(pco, pdev);
+	    code = COS_WRITE_OBJECT(pco, pdev, resourceNone);
 	    if (code < 0)
 		return code;
 	}
