@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdte.c 11608 2010-08-06 11:11:40Z ken $ */
+/* $Id: gdevpdte.c 11993 2011-01-04 10:48:32Z ken $ */
 /* Encoding-based (Type 1/2/42) text processing for pdfwrite. */
 
 #include "math_.h"
@@ -98,13 +98,21 @@ pdf_add_ToUnicode(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_t *pdfon
 	     * Technical Note 5411, 'ToUnicode Mapping File Tutorial'
 	     * page 3.
 	     */
-	    uint num_codes = 256, key_size = 2;
+	    /* Unfortunately, the above is not true. See the PDF Reference (version 1.7
+	     * p 472 'ToUnicode CMaps'. Even that documentation is incorrect as it
+	     * describes codespaceranges, in fact for Acrobat this is irrelevant,
+	     * but the bfranges must be one byte for simple fonts. By altering the
+	     * key size for CID fonts we can write both consistently correct.
+	     */
+	    uint num_codes = 256, key_size = 1;
 	    
 	    if (font->FontType == ft_CID_encrypted) {
 		gs_font_cid0 *pfcid = (gs_font_cid0 *)font;
 
 		num_codes = pfcid->cidata.common.CIDCount;
+		key_size = 2;
 	    } else if (font->FontType == ft_CID_TrueType) {
+		key_size = 2;
 #if 0
 		gs_font_cid2 *pfcid = (gs_font_cid2 *)font;
 
@@ -1017,7 +1025,7 @@ process_text_modify_width(pdf_text_enum_t *pte, gs_font *font,
 	    }
 	}
 	pdf_char_widths_to_uts(pdfont3, &cw); /* convert design->text space */
-	if (pte->text.operation & TEXT_DO_DRAW) {
+	if (pte->text.operation & (TEXT_DO_DRAW | TEXT_RENDER_MODE_3)) {
 	    gs_distance_transform(cw.Width.xy.x * ppts->values.size,
 				  cw.Width.xy.y * ppts->values.size,
 				  &ppts->values.matrix, &did);
