@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtf.c 11681 2010-09-02 13:50:13Z ken $ */
+/* $Id: gdevpdtf.c 11770 2010-10-07 12:49:54Z ken $ */
 /* Font and CMap resource implementation for pdfwrite text */
 #include "memory_.h"
 #include "gx.h"
@@ -209,7 +209,6 @@ find_std_appearance(const gx_device_pdf *pdev, gs_font_base *bfont,
 	return -1;
     }
 
-    mask |= FONT_SAME_OUTLINES;
     for (i = 0; i < PDF_NUM_STANDARD_FONTS; ++psf, ++i) {
 	gs_font_base *cfont;
 	int code;
@@ -636,11 +635,11 @@ has_extension_glyphs(gs_font *pfont)
     const int sl = strlen(gx_extendeg_glyph_name_separator);
 
     psf_enumerate_glyphs_begin(&genum, (gs_font *)pfont, NULL, 0, GLYPH_SPACE_NAME);
-    for (glyph = gs_no_glyph; (code = psf_enumerate_glyphs_next(&genum, &glyph)) != 1; ) {
+    for (glyph = gs_no_glyph; (psf_enumerate_glyphs_next(&genum, &glyph)) != 1; ) {
 	code = pfont->procs.glyph_name(pfont, glyph, &str);
 	if (code < 0)
 	    return code;
-	l = str.size - sl, j;
+	l = str.size - sl;
 	for (j = 0; j < l; j ++)
 	    if (!memcmp(gx_extendeg_glyph_name_separator, str.data + j, sl))
 		return true;
@@ -895,6 +894,8 @@ pdf_font_simple_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 					   pdf_font_descriptor_FontType(pfd),
 					   pdf_write_contents_simple);
 
+    if (code < 0)
+	return(gs_note_error(code));
     pdfont->FontDescriptor = pfd;
     set_is_MM_instance(pdfont, pdf_font_descriptor_font(pfd, false));
     *ppfres = pdfont;
