@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevpsd.c 11346 2010-06-01 17:41:20Z mvrhel $ */
+/*$Id: gdevpsd.c 12306 2011-03-23 21:34:20Z mvrhel $ */
 /* PhotoShop (PSD) export device, supporting DeviceN color models. */
 
 #include "math_.h"
@@ -152,6 +152,9 @@ RELOC_PTRS_END
 static void
 psd_device_finalize(void *vpdev)
 {
+    /* We need to deallocate the compressed_color_list.  
+       and the names. */
+    devn_free_params((gx_device*) vpdev);
     gx_device_finalize(vpdev);
 }
 
